<?php
/**
 * Factory for record driver data formatting view helper
 *
 * PHP version 5
 *
 * Copyright (C) Villanova University 2016.
 * Copyright (C) The National Library of Finland 2017.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * @category VuFind
 * @package  View_Helpers
 * @author   Demian Katz <demian.katz@villanova.edu>
 * @author   Konsta Raunio <konsta.raunio@helsinki.fi>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org/wiki/development:architecture:record_data_formatter
 * Wiki
 */
namespace fid_bbi\View\Helper\Root;

use VuFind\View\Helper\Root\RecordDataFormatter;

/**
 * Factory for record driver data formatting view helper
 *
 * @category VuFind
 * @package  View_Helpers
 * @author   Demian Katz <demian.katz@villanova.edu>
 * @author   Konsta Raunio <konsta.raunio@helsinki.fi>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org/wiki/development:architecture:record_data_formatter
 * Wiki
 */
class RecordDataFormatterFactory extends \fid\View\Helper\Root\RecordDataFormatterFactory
{
    /**
     * Get default specifications for displaying data in core metadata.
     *
     * @return array
     */
    public function getAiCoreSpecs()
    {
        $spec = new RecordDataFormatter\SpecBuilder();
        $spec->setTemplateLine(
            'Journal Title', 'getContainerTitle', 'data-containerTitle.phtml'
        );
        $spec->setTemplateLine(
            'Authors/Corporations',
            'getDeduplicatedAuthors',
            'data-authors.phtml',
            [
                'useCache' => true,
                'context' => [
                    'requiredDataFields' => [
                        ['name' => 'role', 'prefix' => 'CreatorRoles::']
                    ],
                    'separator' => '; '
                ]
            ]
        );
        $spec->setLine(
            'Edition', 'getEdition', null,
            ['prefix' => '<span property="bookEdition">', 'suffix' => '</span>']
        );
        $spec->setTemplateLine(
            'In',
            'getJTitle',
            'data-jTitle.phtml'
        );
        $spec->setLine(
            'Format', 'getFormats', 'RecordHelper',
            ['helperMethod' => 'getFormatList']
        );
        $spec->setTemplateLine(
            'Language', 'getLanguages', 'data-transEscCommaSepLang.phtml'
        );
        $spec->setTemplateLine(
            'Published',
            'getPublicationDetails',
            'data-publicationDetails.phtml'
        );
        $spec->setLine(
            'Series', 'getSeries', null, ['recordLink' => 'series']
        );
        $spec->setTemplateLine(
            'Subjects', 'getAllSubjectHeadings', 'data-allSubjectHeadings.phtml'
        );
        $spec->setTemplateLine(
            'child_records', 'getChildRecordCount', 'data-childRecords.phtml',
            ['allowZero' => false]
        );
        /* finc: remove 'Online Access' in record-details #13770 - VE */
        /* $spec->setTemplateLine('Online Access', true, 'data-onlineAccess.phtml'); */
        $spec->setTemplateLine(
            'Related Items', 'getAllRecordLinks', 'data-allRecordLinks.phtml'
        );
        $spec->setTemplateLine('Tags', true, 'data-tags.phtml',[
            'labelFunction' => function() {return null;}
        ]);
        return $spec->getArray();
    }

    /**
     * Get default specifications for displaying data in collection-info metadata.
     *
     * @return array
     */
    public function getDefaultCollectionInfoSpecs()
    {
        $spec = new RecordDataFormatter\SpecBuilder();
        $spec->setLine(
            'Title', 'getTitleDetails'
        );
        $spec->setTemplateLine(
            'Authors/Corporations',
            'getDeduplicatedAuthors',
            'data-authors.phtml',
            [
                'useCache' => true,
                'context' => [
                    'requiredDataFields' => [
                        ['name' => 'role', 'prefix' => 'CreatorRoles::']
                    ],
                    'separator' => '; '
                ]
            ]
        );
        $spec->setLine(
            'Dates of publication', 'getDateSpan'
        );
        $spec->setLine(
            'Summary', 'getSummary'
        );
        $spec->setTemplateLine(
            'Published', 'getPublicationDetails', 'data-publicationDetails.phtml'
        );
        $spec->setLine(
            'Edition', 'getEdition', null,
            ['prefix' => '<span property="bookEdition">', 'suffix' => '</span>']
        );
        $spec->setTemplateLine(
            'Subjects', 'getAllSubjectHeadings', 'data-allSubjectHeadings.phtml'
        );
        /* finc: remove 'Online Access' in record-details #13770 - VE */
        /* $spec->setTemplateLine('Online Access', true, 'data-onlineAccess.phtml'); */
        $spec->setLine(
            'Item Description', 'getGeneralNotes'
        );
        $spec->setLine(
            'Production Credits',
            'getProductionCredits'
        );
        $spec->setTemplateLine(
            'Set Multipart',
            'getHierarchyParentTitle',
            'data-hierarchyParentTitle.phtml'
        );
        $spec->setTemplateLine(
            'ISBN', 'getISBNs', 'data-isbn.phtml'
        );
        $spec->setTemplateLine(
            'ISSN', 'getISSNs', 'data-issn.phtml'
        );
        $spec->setLine(
            'Notes',
            'getAdditionalNotes',
            null,
            [
                'useCache' => true
            ]
        );
        $spec->setLine(
            'Format', 'getFormats', 'RecordHelper',
            ['helperMethod' => 'getFormatList']
        );
        $spec->setTemplateLine(
            'Language', 'getLanguages', 'data-transEscCommaSepLang.phtml'
        );
        $spec->setMultiLine(
            'Additionals',
            'getAdditionals',
            [$this,'additionals']
        );
        return $spec->getArray();
    }

    /**
     * Get default specifications for displaying data in collection-record metadata.
     *
     * @return array
     */
    public function getDefaultCollectionRecordSpecs()
    {
        $spec = new RecordDataFormatter\SpecBuilder();
        $spec->setLine('Summary', 'getSummary');
        $spec->setTemplateLine(
            'Authors/Corporations',
            'getDeduplicatedAuthors',
            'data-authors.phtml',
            [
                'useCache' => true,
                'context' => [
                    'requiredDataFields' => [
                        ['name' => 'role', 'prefix' => 'CreatorRoles::']
                    ],
                    'separator' => '; '
                ]
            ]
        );
        $spec->setTemplateLine(
            'Language', 'getLanguages', 'data-transEscCommaSepLang.phtml'
        );
        $spec->setLine(
            'Format', 'getFormats', 'RecordHelper',
            ['helperMethod' => 'getFormatList']
        );
        $spec->setLine(
            'Access', 'getAccessRestrictions'
        );
        $spec->setLine(
            'Related Items', 'getRelationshipNotes'
        );
        return $spec->getArray();
    }

    /**
     * Get default specifications for displaying data in core metadata.
     *
     * @return array
     */
    public function getDefaultCoreSpecs()
    {
        $spec = new RecordDataFormatter\SpecBuilder();
        $spec->setTemplateLine(
            'Published in', 'getContainerTitle', 'data-containerTitle.phtml'
        );
        $spec->setLine(
            'New Title', 'getNewerTitles', null, ['recordLink' => 'title']
        );
        $spec->setLine(
            'Previous Title', 'getPreviousTitles', null, ['recordLink' => 'title']
        );
        $spec->setLine(
            'Title', 'getTitleDetails'
        );
        $spec->setTemplateLine(
            'Authors/Corporations', 'getDeduplicatedAuthors', 'data-authors.phtml',
            [
                'useCache' => true,
                'context' => [
                    'requiredDataFields' => [
                        ['name' => 'role', 'prefix' => 'CreatorRoles::']
                    ],
                    'separator' => '; '
                ]
            ]
        );
        $spec->setTemplateLine(
            'Title Uniform', 'getTitleUniform', 'data-titleUniform.phtml',
            [
                'labelFunction' => [$this,'titleUniform']
            ]
        );
        $spec->setLine(
            'Edition', 'getEdition', null,
            ['prefix' => '<span property="bookEdition">', 'suffix' => '</span>']
        );
        $spec->setLine(
            'Dissertation Note',
            'getDissertationNote',
            null,
            ['separator' => ', ']
        );
        $spec->setLine(
            'Format', 'getFormats', 'RecordHelper',
            ['helperMethod' => 'getFormatList']
        );
        $spec->setTemplateLine(
            'Language', 'getLanguages', 'data-transEscCommaSepLang.phtml'
        );
        $spec->setTemplateLine(
            'Published',
            'getPublicationDetails',
            'data-publicationDetails.phtml'
        );
        $spec->setTemplateLine(
            'Set Multipart',
            'getHierarchyParentTitle',
            'data-hierarchyParentTitle.phtml'
        );
        $spec->setTemplateLine(
            'Subjects', 'getAllSubjectHeadings', 'data-allSubjectHeadings.phtml'
        );
        $spec->setTemplateLine(
            'child_records', 'getChildRecordCount', 'data-childRecords.phtml',
            ['allowZero' => false]
        );
        /* finc: remove 'Online Access' in record-details #13770 - VE */
        /* $spec->setTemplateLine('Online Access', true, 'data-onlineAccess.phtml'); */
        $spec->setTemplateLine(
            'Related Items', 'getAllRecordLinks', 'data-allRecordLinks.phtml'
        );
        $spec->setMultiLine(
            'Additionals',
            'getAdditionals',
            [$this,'additionals']
        );
        $spec->setLine(
            'Source',
            'getMegaCollection'
        );
        $spec->setMultiLine(
            'Other Relationship Entry',
            'getOtherRelationshipEntry',
            [$this,'otherRelationShipEntry']
        );
        $spec->setLine(
            'Notes',
            'getAdditionalNotes',
            null,
            [
                'useCache' => true
            ]
        );
        $spec->setTemplateLine('Tags', true, 'data-tags.phtml',[
            'labelFunction' => function() {return null;}
        ]);
        return $spec->getArray();
    }

    /**
     * Get default specifications for displaying data in the description tab.
     *
     * @return array
     */
    public function getDefaultDescriptionSpecs()
    {
        $spec = new RecordDataFormatter\SpecBuilder();
        $spec->setLine(
            'Summary',
            'getSummary'
        );
        $spec->setLine(
            'Physical Description',
            'getPhysicalDescriptions'
        );
        $spec->setLine(
            'Format', 'getSystemDetails'
        );
        $spec->setTemplateLine(
            'Language', 'getLanguages', 'data-transEscCommaSepLang.phtml'
        );
        $spec->setTemplateLine(
            'Subjects', 'getAllSubjectHeadingsExtended', 'data-allSubjectHeadingsExtended.phtml'
        );
        /*
        $spec->setTemplateLine(
            'RVK', 'getRVK', 'data-rvk.phtml'
        );
        */
        $spec->setMultiLine(
            'RVK',
            true,
            [$this,'multiRvk']
        );
        $spec->setTemplateLine(
            'BK', 'getBK', 'data-bk.phtml'
        );
        $spec->setTemplateLine(
            'DDC', 'getDDC', 'data-ddc.phtml'
        );
        $spec->setLine(
            'Bibliography',
            'getBibliographyNotes'
        );
        $spec->setTemplateLine(
            'ISBN', 'getISBNs', 'data-isbn.phtml'
        );
        $spec->setTemplateLine(
            'ISSN', 'getISSNs', 'data-issn.phtml'
        );
        $spec->setLine(
            'DOI',
            'getCleanDOI'
        );
        $spec->setLine(
            'EISSN',
            'getEISSNs'
        );

        return $spec->getArray();
    }

    /**
     * Get marc specifications for displaying data in core metadata.
     *
     * @return array
     */
    public function getMarcCoreSpecs()
    {
        $spec = new RecordDataFormatter\SpecBuilder();
        $spec->setLine(
            'Title', 'getTitle'
        );

        $spec->setLine(
            'In','getTitlePart'
        );
        $spec->setLine(
            'Published in','getMarcContainerTitle',
            null,
            ['separator' => ', ']
        );
        $spec->setTemplateLine(
            'Authors/Corporations',
            'getDeduplicatedAuthors',
            'data-authors.phtml',
            [
                'useCache' => true,
                'context' => [
                    'requiredDataFields' => [
                        ['name' => 'role', 'prefix' => 'CreatorRoles::']
                    ],
                    'separator' => '; '
                ]
            ]
        );
        $spec->setTemplateLine(
            'Series', 'getSeries', 'data-series.phtml'
        );
        $spec->setTemplateLine(
            'Published',
            'getPublicationDetails',
            'data-publicationDetails.phtml'
        );
        $spec->setLine(
            'Year of Publication',
            'getPublishDateSort'
        );
        $spec->setLine(
            'Physical Description',
            'getPhysicalDescriptions'
        );
        $spec->setLine(
            'Format', 'getFormats', 'RecordHelper',
            ['helperMethod' => 'getFormatList']
        );
        $spec->setLine(
            'Source',
            'getMegaCollection'
        );
        $spec->setTemplateLine('Tags', true, 'data-tags.phtml',[
            'labelFunction' => function() {return null;}
        ]);
        return $spec->getArray();
    }

    /**
     * return render details for rvk data
     * uses two different functions
     * @return callback
     */
    public function multiRvk($data, $options, $driver)
    {
        $args = func_get_args();
        // Sort the data:
        $final = [];
        $data = $driver->tryMethod('getRvkWithMetadata');
        if (!empty($data)) {
            return [[
                'label' => 'RVK',
                'values' => $data,
                'options' => [
                    'pos' => $options['pos'],
                    'renderType' => 'RecordDriverTemplate',
                    'template' => 'data-rvkNotation.phtml',
                ],
            ]];
        }
        $data = $driver->tryMethod('getRvk');
        if (!empty($data)) {
            return [[
                'label' => 'RVK',
                'values' => $data,
                'options' => [
                    'pos' => $options['pos'],
                    'renderType' => 'RecordDriverTemplate',
                    'template' => 'data-rvk.phtml',
                ],
            ]];
        }
        return [];
    }

}
